<?php
/**
 * This file is part of PHP Mess Detector.
 *
 * Copyright (c) Manuel Pichler <mapi@phpmd.org>.
 * All rights reserved.
 *
 * Licensed under BSD License
 * For full copyright and license information, please see the LICENSE file.
 * Redistributions of files must retain the above copyright notice.
 *
 * @author Manuel Pichler <mapi@phpmd.org>
 * @copyright Manuel Pichler. All rights reserved.
 * @license https://opensource.org/licenses/bsd-license.php BSD License
 * @link http://phpmd.org/
 */

class testRuleAppliesToVariablesWithinForeach
{
    public function testing()
    {
        $abc = 1;

        foreach ([1, 2, 3] as $i) {
            $a = $i;
            echo $a;
            foreach ([4, 5, 6] as $k => $v) {
                echo $k;
                echo $v;
                $x = (string) $k . (string) $v;
            }
        }
    }
}
